//
//  DataStore.m
//  VideoGames
//
//  Created by Brian Moakley on 6/8/14.
//  Copyright (c) 2014 Razeware. All rights reserved.
//

@import CoreData;
#import "DataStore.h"
#import "SAXParser.h"
#import "RWTReviewer.h"
#import "RWTVideoGame.h"
#import "VideoGame.h"
#import "Reviewer.h"

@interface DataStore()

@property (strong, nonatomic) NSManagedObjectContext * managedObjectContext;
@property (strong, nonatomic) NSPersistentStoreCoordinator * persistentStoreCoordinator;
@property (strong, nonatomic) NSManagedObjectModel * managedObjectModel;

@end


@implementation DataStore

-(instancetype) init
{
    self = [super init];
    if (self)
    {
        _games = [[NSMutableArray alloc] init];
        _reviewers = [[NSMutableArray alloc] init];
    }
    return self;
}

+(instancetype) sharedInstance
{
    static DataStore * sharedstore = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedstore = [[self alloc] init];
    });
    return sharedstore;
}

-(void) loadLibrary
{
    NSFetchRequest * fetchRequest = [[NSFetchRequest alloc] init];
    fetchRequest.entity = [NSEntityDescription insertNewObjectForEntityForName:@"Reviewer" inManagedObjectContext:self.managedObjectContext];
    self.reviewers = [[self.managedObjectContext executeFetchRequest:fetchRequest error:nil] mutableCopy];
    
    fetchRequest.entity = [NSEntityDescription insertNewObjectForEntityForName:@"VideoGame" inManagedObjectContext:self.managedObjectContext];
    self.games = [[self.managedObjectContext executeFetchRequest:fetchRequest error:nil] mutableCopy];
}

-(void) importLibrary
{
    SAXParser * parser = [[SAXParser alloc] init];
    NSString * xmlPath = [[NSBundle mainBundle] pathForResource:@"video_games" ofType:@"xml"];
    NSURL * xmlUrl = [NSURL fileURLWithPath:xmlPath];
    
    parser.xml = [NSString stringWithContentsOfURL:xmlUrl encoding:NSUTF8StringEncoding error:nil];
    
    NSDictionary * parsedResults = [parser parseFeed];
    NSArray * reviewerList= parsedResults[@"reviewers"];
    
    for (RWTReviewer * reviewer in reviewerList)
    {
        Reviewer * newReviewer = [NSEntityDescription insertNewObjectForEntityForName:@"Reviewer" inManagedObjectContext:self.managedObjectContext];
        newReviewer.name = reviewer.name;
        newReviewer.bio = reviewer.bio;
        
        for (RWTVideoGame * game in reviewer.reviews)
        {
            VideoGame * newGame = [NSEntityDescription insertNewObjectForEntityForName:@"VideoGame" inManagedObjectContext:self.managedObjectContext];
            newGame.name = game.name;
            newGame.genre = game.genre;
            newGame.synopsis = game.synopsis;
            newGame.rating = @(game.rating);
            
            newGame.reviwer = newReviewer;
            [newReviewer addReviewedGamesObject:newGame];
            [self.games addObject:newGame];
        }
        [self.reviewers addObject:newReviewer];
    }
    
    NSError * error;
    if (![self.managedObjectContext save:&error])
    {
        NSLog(@"Error: %@", error);
    }
}

-(NSManagedObjectContext *) managedObjectContext
{
    if (_managedObjectContext != nil)
    {
        return _managedObjectContext;
    }
    if (self.persistentStoreCoordinator != nil)
    {
        _managedObjectContext = [[NSManagedObjectContext alloc] init];
        _managedObjectContext.persistentStoreCoordinator = self.persistentStoreCoordinator;
    }
    return _managedObjectContext;
}

-(NSPersistentStoreCoordinator *) persistentStoreCoordinator
{
    if (_persistentStoreCoordinator != nil)
    {
        return _persistentStoreCoordinator;
    }
    NSFileManager * fileManager = [NSFileManager defaultManager];
    NSURL * documentDirectory = [fileManager URLForDirectory:NSDocumentDirectory inDomain:NSUserDomainMask appropriateForURL:nil create:NO error:nil];
    NSURL * storeFile = [documentDirectory URLByAppendingPathComponent:@"videogames.sqlite"];
    
    NSError * error = nil;
    _persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:self.managedObjectModel];
    if (![_persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeFile options:nil error:&error])
    {
        NSLog(@"Error: %@", error);
    }
    return _persistentStoreCoordinator;
}


-(NSManagedObjectModel *) managedObjectModel
{
    if (_managedObjectModel != nil)
    {
        return _managedObjectModel;
    }
    NSURL * modelURL = [[NSBundle mainBundle] URLForResource:@"VideoGamesModel" withExtension:@"momd"];
    _managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    return _managedObjectModel;
}


@end
